from django.db import models
from django.contrib.auth.models import (
    BaseUserManager, AbstractBaseUser
)
from salesUnit.models import SalesUnit

class UserManager(BaseUserManager):
    def create_user(self, unit, identify, email, shenfen, password=None,is_staff=False, is_active=True, is_admin=False):
        """
        Creates and saves a User with the given email and password.
        """
        if not email:
            raise ValueError('Users must have an email address')
        if not password:
            raise ValueError('Users must have an password')
        if not identify:
            raise ValueError('Users must have an identify')
        if not unit:
            raise ValueError('Users must have an unit')
        user = self.model(
            email=self.normalize_email(email),
        )
        user.shenfen = shenfen
        user.staff = is_staff
        user.active = is_active
        user.admin = is_admin
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, unit, identify, email, password):
        """
        Creates and saves a superuser with the given email and password.
        """
        user = self.create_user(
            unit,
            identify,
            email,
            password=password,
            shenfen ='admin',
            is_staff=True,
            is_admin=True
        )
        return user

class User(AbstractBaseUser):
    email  = models.EmailField(
        verbose_name='email address',
        max_length=255,
        unique=True,
    )
    identify = models.CharField(
        max_length=255,
        unique=True,
        null=False)

    shenfen_choice = (
        ('salesAssistant', '業務助理' ),
        ('multiUnitSalesAssistant', '多單位業務助理'),
        ('salesHumanResources', '業務人資'),
        ('humanResources', '總公司人資' ),
        ('catchdog', '使用者'),
        )

    shenfen = models.CharField(
        max_length=30,
        choices=shenfen_choice,
        null=True)

    full_name = models.CharField(max_length=255, blank=True, null=True)
    unit = models.ManyToManyField(SalesUnit, blank=True)
    timeStamp = models.DateTimeField(auto_now_add=True)
    admin  = models.BooleanField(default=False) # a superuser
    staff  = models.BooleanField(default=False) # a staff user
    active = models.BooleanField(default=True)
    # notice the absence of a "Password field", that's built in.
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['unit', 'identify'] # Email & Password are required by default.

    objects = UserManager()

    def save(self, **kwargs):
        super(User, self).save(**kwargs)
        if not getattr(self, 'profile', None):
            Profile.objects.create(user=self)
    def get_full_name(self):
        # The user is identified by their email address
        return self.full_name

    def get_short_name(self):
        # The user is identified by their email address
        return self.full_name

    def get_identify(self):
        return self.identify

    def get_unit(self):
        return self.unit

    def __str__(self):              # __unicode__ on Python 2
        return self.email

    def has_perm(self, perm, obj=None):
        #"Does the user have a specific permission?"
        # Simplest possible answer: Yes, always
        return True

    def has_module_perms(self, app_label):
        #"Does the user have permissions to view the app `app_label`?"
        # Simplest possible answer: Yes, always
        return True

    def get_shenfen(self):
        return self.shenfen

    @property
    def is_staff(self):
        return self.staff

    @property
    def is_admin(self):
        #"Is the user a admin member?"
        return self.admin

    @property
    def is_active(self):
        #"Is the user active?"
        return self.active

    # def user_permissions(self):
    #     permissions = {
    #         'Active': self.active,
    #         'Staff': self.staff,
    #         'SalesAssistant': self.salesAssistant,
    #         'HumanResources': self.humanResources,
    #         'SalesHumanResources': self.salesHumanResources,
    #         'Admin': self.admin
    #     }
    #     return permissions

class Profile(models.Model):
    user = models.OneToOneField(User)
    updated = models.DateTimeField(null=True, blank=True)
    pswd = models.CharField(
        max_length=255, null=True, blank=True)
    pswd2 = models.CharField(
        max_length=255, null=True, blank=True)
