from rest_framework import serializers

from django.utils.timezone import now
from django.db.transaction import atomic

from accounts.models import User
from salesUnit.models import SalesUnit
from log.models import LogForUserBehavior
from .models import Application
from sales.models import Sales
from tempSales.models import TempSales
from modified.models import modified
from notification.models import Notification
from datetime import datetime
import requests
import os
import random
import string
import threading

def get_sales_unit_detail(url):
    result = []
    # url = 'http://stagingerp.smartstudy.tw/api/v1/sales'
    for unit in SalesUnit.objects.filter().exclude(unit_name="看不到"):
        temp = {
            "unit_num": unit.unit_num,
            "unit_name": unit.unit_name,
            "sales": []
        }
        for sales in unit.sales_set.filter(): #(state="在職"):
            this_sales = {
                "id": sales.id,
                "name": sales.full_name,
                "phone": sales.cellphone_one,
                "type": sales.work_type,
                "state": sales.state,
                "company_name": "",
                "company_id": "",
                "company_number": ""
            }
            if sales.company:
                this_sales["company_name"] = sales.company.company_name
                this_sales["company_id"] = sales.company.id # django
                this_sales["company_number"] = sales.company.company_id # company id
            if sales.date_of_birth:
                this_sales['birthday'] = sales.date_of_birth.strftime('%Y%d%m')
            temp['sales'].append(this_sales)
        result.append(temp)
    res = requests.post(url=url, json={
        "sales": result
    })
    print(res.content, res.status_code, "test", url)


def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))

class ApplicationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Application
        fields = '__all__'

    @atomic
    def update(self, instance, validated_data):
        user_id = self.context['request'].user.id
        if validated_data.get('status', instance.status) in ['待補紙本', '待審退回', '紙本有誤', '申請完成']:
            notification = Notification()
            notification.application = instance
            notification.sales_name = instance.tempSales.full_name
            notification.sales_id = instance.tempSales.sales_id
            notification.unit = instance.tempSales.sales_unit.display_name
            if instance.tempSales.company:
                notification.company = instance.tempSales.company.company_name
            else:
                notification.company = None
            notification.sales_type = instance.tempSales.work_type
            notification.sales_position = instance.tempSales.position.title
            if instance.status in ["待審退回", "紙本有誤"]:
                if instance.explainText != validated_data.get('explainText', instance.explainText):
                    notification.app_status = "說明更改"
                else:
                    notification.app_status = validated_data.get('status', instance.status)
            else:
                notification.app_status = validated_data.get('status', instance.status)
            notification.save()
        instance.explainText = validated_data.get('explainText', instance.explainText)
        instance.status = validated_data.get('status', instance.status)
        instance.save()
        logForUserBehavior = LogForUserBehavior()
        setattr(logForUserBehavior, 'behavior', instance.use + "-" + instance.status)
        setattr(logForUserBehavior, 'application', instance)
        setattr(logForUserBehavior, 'user', User.objects.get(id=user_id))
        setattr(logForUserBehavior, 'full_name', User.objects.get(id=user_id).full_name)
        logForUserBehavior.save()
        if instance.status == "申請完成":
            tempSales = instance.tempSales
            new_modified = modified()
            if instance.use == "新增業務":
                sales = Sales()
                for i in tempSales.__dict__:
                    if (i != 'id') and (i !='sales_id') and (i !='_state') and ('TempSales' not in i):
                        setattr(sales, i, getattr(tempSales,i))
                    setattr(sales, 'date_of_entry', datetime.now())
                inUnit = Sales.objects.filter(sales_unit=tempSales.sales_unit.id)
                if not inUnit:
                    sales.sales_id = tempSales.sales_unit.unit_num + "001"
                else:
                    sales.sales_id = tempSales.sales_unit.unit_num + (str)((int)(inUnit.order_by('id').last().sales_id[-3:])+1).zfill(3)
                sales.save()
                new_modified.user = instance.user
                new_modified.sales_unit = sales.sales_unit.display_name
                new_modified.change = "入職"
                new_modified.sales_id = sales.sales_id
                new_modified.sales_name = sales.full_name
                new_modified.orignal_sales = sales
                new_modified.position = sales.position.title
                new_modified.explainText = "入職"
                new_modified.work_type = sales.work_type
                if not new_modified.created:
                    new_modified.created = datetime.now()
                if sales.company:
                    new_modified.company = sales.company.company_name
                else:
                    new_modified.company = None
                new_modified.save()
            elif instance.use == "修改資料":
                new_modified = modified()
                origin_user = instance.origin_user
                change = ""
                if tempSales.work_type != origin_user.work_type:
                    change = "異動"
                if tempSales.position.rank > origin_user.position.rank:
                    change = "升職"
                elif tempSales.position.rank < origin_user.position.rank:
                    change = "降職"
                if tempSales.state == "離職":
                    change = "離職"
                    new_modified.created = tempSales.date_of_leave
                if change != "":
                    new_modified.user = instance.user
                    new_modified.sales_unit = tempSales.sales_unit.display_name
                    new_modified.change = change
                    new_modified.sales_id = tempSales.sales_id
                    new_modified.sales_name = tempSales.full_name
                    new_modified.orignal_sales = origin_user
                    new_modified.position = tempSales.position.title
                    new_modified.explainText = instance.modifiedText
                    new_modified.work_type = tempSales.work_type
                    if not new_modified.created:
                        new_modified.created = datetime.now()
                    if tempSales.company:
                        new_modified.company = tempSales.company.company_name
                    else:
                        new_modified.company = None
                    new_modified.save()
                for i in tempSales.__dict__:
                    if (i != 'id') and (i != 'sales_id') and (i !='_state') and ('TempSales' not in i) and (i != "date_of_come") and (i != "date_of_entry"):
                        setattr(origin_user, i, getattr(tempSales,i))
                origin_user.save()
            url = "http://stagingerp.smartstudy.tw/api/v1/sales"
            threading.Thread(target = get_sales_unit_detail, args = (url,)).start()
            url = "http://serp.smartstudy.tw/api/v1/sales"
            threading.Thread(target = get_sales_unit_detail, args = (url,)).start()
        return instance

class ApplicationListAPISerializer(serializers.ModelSerializer):
    tempSales__full_name = serializers.CharField(max_length=200)
    tempSales__work_type = serializers.CharField(max_length=200)
    tempSales__sales_id = serializers.CharField(max_length=200)
    tempSales__company__company_name = serializers.CharField(max_length=200)
    tempSales__position__title = serializers.CharField(max_length=200)
    unit__display_name = serializers.CharField(max_length=200)
    class Meta:
        model = Application
        fields = '__all__'



# class TempSalesBankInfoSerializer(serializers.ModelSerializer):
#
#     class Meta:
#         model = TempSales
#         fields = ('bank_num', 'bank_pri_name', 'bank_sub_name', 'id', 'full_name', 'bank_account')
    # def create(self, validated_data):
    #     print(validated_data)
    #     test = Test(
    #             created = validated_data.get('created', None),
    #             user = validated_data.get('user', None),
    #             title = validated_data.get('title', None),
    #             img1 = validated_data.get('img1', None),
    #             img1_thumb = validated_data.get('img1_thumb', None),
    #             img1_datetime = validated_data.get('img1_datetime', None),
    #             img2 = validated_data.get('img2', None),
    #             img2_datetime = validated_data.get('img2_datetime', None),
    #         )
    #     test.save(force_insert=True ,force_update=True, api='Create')
    #     return test
    #
    # def update(self, instance, validated_data):
    #     print(validated_data)
    #     instance.user = validated_data.get('user', instance.user)
    #     instance.title = validated_data.get('title', instance.title)
    #     instance.img1 = validated_data.get('img1', instance.img1)
    #     instance.img1_thumb = validated_data.get('img1_thumb', instance.img1_thumb)
    #     instance.img1_datetime = validated_data.get('img1_datetime', instance.img1_datetime)
    #     instance.img2 = validated_data.get('img2', instance.img2)
    #     instance.img2_datetime = validated_data.get('img2_datetime', instance.img2_datetime)
    #     instance.save(force_insert=False ,force_update=True, api='Update')
    #     return instance
