from django.shortcuts import render
from rest_framework import generics, viewsets
from rest_framework.permissions import IsAuthenticated
from tempSales.models import TempSales
from salesUnit.models import SalesUnit
from sales.models import Sales
from .models import Application
from accounts.models import User
from .serializers import ApplicationSerializer, ApplicationListAPISerializer
# Create your views here.
class ApplicationViewSet(viewsets.ModelViewSet):
    queryset = Application.objects.all()
    serializer_class = ApplicationSerializer
    permission_classes = (IsAuthenticated,)

class ApplicationListAPI(generics.ListAPIView):
    serializer_class = ApplicationListAPISerializer
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        """
        Optionally restricts the returned purchases to a given user,
        by filtering against a `username` query parameter in the URL.
        """
        queryset = Application.objects.all()
        # serializer = ApplicationSerializer(queryset, many=True)
        unit = self.request.GET.get("unit")
        status = self.request.GET.get("status")
        status_opt = (
                ('編輯中', '編輯中'), #0
                ('待審核', '待審核'), #1
                ('待補紙本', '待補紙本'), #2
                ('待審退回', '待審退回'), #3
                ('紙本有誤', '紙本有誤'), #4
                ('申請完成', '申請完成'), #5
                ('刪除', '刪除') #6
            )
        if unit and status:
            queryset = queryset.filter(unit=(int(unit)))
            if status == "others":
                queryset = queryset.exclude(status="申請完成")
                queryset = queryset.exclude(status="編輯中")
                queryset = queryset.exclude(status="刪除")
            else:
                queryset = queryset.filter(status=status_opt[(int)(status)][0])
            queryset = queryset.select_related()
            output = []
            for i in queryset.values(
                "user",
                "id",
                "timeStamp",
                "updated",
                "status",
                "use",
                "explainText",
                "modifiedText",
                "origin_user",
                "unit",
                "tempSales_id",
                "tempSales__full_name",
                "tempSales__work_type",
                "tempSales__sales_id",
                "tempSales__company__company_name",
                "tempSales__position__title",
                "unit__display_name"):
                output.append(
                    {
                        "user": User.objects.filter(id=i["user"]).first() or None,
                        "id" : i["id"],
                        "timeStamp" : i["timeStamp"],
                        "updated" : i["updated"],
                        "status" : i["status"],
                        "use" : i["use"],
                        "origin_user": Sales.objects.filter(id=i["origin_user"]).first() or None,
                        "explainText" : i["explainText"],
                        "modifiedText" : i["modifiedText"],
                        "unit" : SalesUnit.objects.get(id=i["unit"]),
                        "tempSales" : TempSales.objects.get(id=i["tempSales_id"]),
                        "tempSales__full_name" : i["tempSales__full_name"],
                        "tempSales__work_type" : i["tempSales__work_type"],
                        "tempSales__sales_id" : i["tempSales__sales_id"],
                        "tempSales__company__company_name" : i["tempSales__company__company_name"],
                        "tempSales__position__title" : i["tempSales__position__title"],
                        "unit__display_name" : i["unit__display_name"]
                    }
                )
            return output
        else:
            return []
