from django.db import models
from django.conf import settings
from django.utils import timezone
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.files.base import ContentFile

from datetime import date
from PIL import Image
from io import BytesIO
import os, random

from salesUnit.models import SalesUnit
from position.models import Position
from company.models import Company
# Create your models here.

imageWithModelName = 'image/Sales/'

def get_filename_ext(filepath):
	base_name = os.path.basename(filepath)
	name, ext = os.path.splitext(base_name)
	return name, ext

class CustomImageField(models.ImageField):
	def __init__(self, contract_type=None, *args, **kwargs):
		self.contract_type = contract_type
		super(CustomImageField, self).__init__(*args, **kwargs)

class Sales(models.Model):
    def upload_image_path(self, filename, *arg, **kwargs):
        new_filename = str(random.randint(1,3910209312))
        name, ext = get_filename_ext(filename)
        final_filename = '{new_filename}{ext}'.format(new_filename=new_filename, ext=ext)
        return "pdf/sales/{new_filename}/{final_filename}".format(
				new_filename=self.sales_id,
				final_filename=final_filename
				)
    full_name = models.CharField(max_length=255, blank=True, null=True)

    state_choice = (
    	('離職', '離職'),
    	('在職', '在職'),)

    state = models.CharField(max_length=10, choices=state_choice, null=True, blank=True)

    sex_choice = (
    	('男', '男'),
    	('女', '女'),
    	('其他', '其他'))

    sex = models.CharField(max_length=10, choices=sex_choice)

    work_type_choice =(
    	('業務','業務'),
    	('電訪','電訪'))

    work_type = models.CharField(
    	max_length=4,
    	choices=work_type_choice)

    sales_id = models.CharField(
        max_length=255,
        unique=True)

    sales_unit = models.ForeignKey(SalesUnit,default=1)

    immediate_supervisor = models.ForeignKey(
        'self',
        blank=True,
        null=True,
        related_name='immediateSupervisor')

    roc_id = models.CharField(
        max_length=10,
        null=False)

    company = models.ForeignKey(
        Company,
        blank=True,
        null=True)

    date_of_birth = models.DateField(null=True, blank=True)
    date_of_entry = models.DateField(null=True, blank=True)
    date_of_come = models.DateField(null=True, blank=True)
    date_of_leave = models.DateField(null=True, blank=True)
    cellphone_one = models.CharField(
        max_length=10,
        null=True,
        blank=True,
		default="0900000000")

    cellphone_two = models.CharField(
        max_length=10,
        null=True,
        blank=True)

    email = models.EmailField(
		max_length=70,
		blank=True,
    )
    residence_vil = models.CharField(
        max_length=10,
        null=True,
        blank=True)   #里
    residence_neighbor = models.CharField(
        max_length=10,
        null=True,
        blank=True)   #鄰
    residence_address_city = models.CharField(
        max_length=10,
        null=True,
        blank=True)
    residence_address_area = models.CharField(
        max_length=10,
        null=True,
        blank=True)
    residence_address_etc = models.CharField(
        max_length=255,
        null=True,
        blank=True)
    residence_address_phone = models.CharField(
        max_length=20,
        null=True,
        blank=True)

    mailing_vil = models.CharField(
        max_length=10,
        null=True,
        blank=True)   #里
    mailing_neighbor = models.CharField(
        max_length=10,
        null=True,
        blank=True)   #鄰
    mailing_address_city = models.CharField(
        max_length=10,
        null=True,
        blank=True)
    mailing_address_area = models.CharField(
        max_length=10,
        null=True,
        blank=True)
    mailing_address_etc = models.CharField(
        max_length=255,
        null=True,
        blank=True)
    mailing_address_phone = models.CharField(
        max_length=20,
        null=True,
        blank=True)

    provide_bank_account = models.BooleanField(
        default=True)

    into_whos_back_account = models.CharField(
        max_length=20,
        null=True,
        blank=True)

    bank_num = models.CharField(
        max_length=10,
        null=True,
        blank=True)
    bank_pri_name = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_sub_name = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_account = models.CharField(
        max_length=100,
        null=True,
        blank=True)
    bank_account_name_2 = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_num_2 = models.CharField(
        max_length=10,
        null=True,
        blank=True)
    bank_pri_name_2 = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_sub_name_2 = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_account_2 = models.CharField(
        max_length=100,
        null=True,
        blank=True)
    bank_account_name_3 = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_num_3 = models.CharField(
        max_length=10,
        null=True,
        blank=True)
    bank_pri_name_3 = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_sub_name_3 = models.CharField(
        max_length=20,
        null=True,
        blank=True)
    bank_account_3 = models.CharField(
        max_length=100,
        null=True,
        blank=True)

    promise_basic_salary = models.DecimalField(
        max_digits=10,
        decimal_places=0,
        default=0)

    promise_start_date = models.DateField(
        blank=True,
        null=True)

    promise_end_date = models.DateField(
        blank=True,
        null=True)

    position = models.ForeignKey(Position,default=1)

    performance_rate = models.DecimalField(
        max_digits=2,
        decimal_places=0,
        default=0)
    organization_rate = models.DecimalField(
        max_digits=2,
        decimal_places=0,
        default=0)
    plus_rate = models.DecimalField(
        max_digits=2,
        decimal_places=0,
        default=0)
    isHuman = models.BooleanField(default=False)

    memo = models.TextField(
		null=True,
		blank=True
	)
    pdfupload = models.FileField(upload_to=upload_image_path,null=True,blank=True)
    # '''    for contract    '''
    # pic_for_contract_1 = CustomImageField(contract_type='contract',null=True,blank=True)
    # pic_for_contract_2 = CustomImageField(contract_type='contract',null=True,blank=True)
    # pic_for_contract_3 = CustomImageField(contract_type='contract',null=True,blank=True)
    # pic_for_contract_4 = CustomImageField(contract_type='contract',null=True,blank=True)
    # pic_for_contract_5 = CustomImageField(contract_type='contract',null=True,blank=True)
    # pic_for_contract_1_thumb = CustomImageField(contract_type='contract.thumb',null=True,blank=True)
    # pic_for_contract_2_thumb = CustomImageField(contract_type='contract.thumb',null=True,blank=True)
    # pic_for_contract_3_thumb = CustomImageField(contract_type='contract.thumb',null=True,blank=True)
    # pic_for_contract_4_thumb = CustomImageField(contract_type='contract.thumb',null=True,blank=True)
    # pic_for_contract_5_thumb = CustomImageField(contract_type='contract.thumb',null=True,blank=True)
    # pic_for_contract_1_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_contract_2_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_contract_3_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_contract_4_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_contract_5_datetime = models.DateTimeField(null=True,blank=True)
	#
    # '''   for basic data    '''
    # pic_for_basic_data_1 = CustomImageField(contract_type='basic_data', null=True,blank=True)
    # pic_for_basic_data_2 = CustomImageField(contract_type='basic_data', null=True,blank=True)
    # pic_for_basic_data_3 = CustomImageField(contract_type='basic_data', null=True,blank=True)
    # pic_for_basic_data_1_thumb = CustomImageField(contract_type='basic_data.thumb', null=True,blank=True)
    # pic_for_basic_data_2_thumb = CustomImageField(contract_type='basic_data.thumb', null=True,blank=True)
    # pic_for_basic_data_3_thumb = CustomImageField(contract_type='basic_data.thumb', null=True,blank=True)
    # pic_for_basic_data_1_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_basic_data_2_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_basic_data_3_datetime = models.DateTimeField(null=True,blank=True)
	#
    # '''    for agent agreement    '''
    # pic_for_agent_agreement_1 = CustomImageField(contract_type='agent_agreement', null=True,blank=True)
    # pic_for_agent_agreement_2 = CustomImageField(contract_type='agent_agreement', null=True,blank=True)
    # pic_for_agent_agreement_1_thumb = CustomImageField(contract_type='agent_agreement.thumb', null=True,blank=True)
    # pic_for_agent_agreement_2_thumb = CustomImageField(contract_type='agent_agreement.thumb', null=True,blank=True)
    # pic_for_agent_agreement_1_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_agent_agreement_2_datetime = models.DateTimeField(null=True,blank=True)
	#
    # '''    for transfer agreement    '''
    # pic_for_transfer_agreement_1 = CustomImageField(contract_type='transfer_agreement', null=True,blank=True)
    # pic_for_transfer_agreement_2 = CustomImageField(contract_type='transfer_agreement', null=True,blank=True)
    # pic_for_transfer_agreement_1_thumb = CustomImageField(contract_type='transfer_agreement.thumb', null=True,blank=True)
    # pic_for_transfer_agreement_2_thumb = CustomImageField(contract_type='transfer_agreement.thumb', null=True,blank=True)
    # pic_for_transfer_agreement_1_datetime = models.DateTimeField(null=True,blank=True)
    # pic_for_transfer_agreement_2_datetime = models.DateTimeField(null=True,blank=True)
	#
    # '''    for position change    '''
    # pic_for_position_change = CustomImageField(contract_type='position_change', null=True,blank=True)
    # pic_for_position_change_thumb = CustomImageField(contract_type='position_change.thumb', null=True,blank=True)
	#
    # pic_for_position_change_datetime = models.DateTimeField(null=True,blank=True)
	#
    # '''    for quit job    '''
    # pic_for_quit = CustomImageField(contract_type='quit', null=True,blank=True)
    # pic_for_quit_thumb = CustomImageField(contract_type='quit.thumb', null=True,blank=True)
	#
    # pic_for_quit_datetime = models.DateTimeField(null=True,blank=True)

    # def __init__(self, *args, **kwargs):
        # super(Sales, self).__init__(*args, **kwargs)
        # self.__original_pic_for_contract_1 = self.pic_for_contract_1
        # self.__original_pic_for_contract_2 = self.pic_for_contract_2
        # self.__original_pic_for_contract_3 = self.pic_for_contract_3
        # self.__original_pic_for_contract_4 = self.pic_for_contract_4
        # self.__original_pic_for_contract_5 = self.pic_for_contract_5
        # self.__original_pic_for_basic_data_1 = self.pic_for_basic_data_1
        # self.__original_pic_for_basic_data_2 = self.pic_for_basic_data_2
        # self.__original_pic_for_basic_data_3 = self.pic_for_basic_data_3
        # self.__original_pic_for_agent_agreement_1 = self.pic_for_agent_agreement_1
        # self.__original_pic_for_agent_agreement_2 = self.pic_for_agent_agreement_2
        # self.__original_pic_for_transfer_agreement_1 = self.pic_for_transfer_agreement_1
        # self.__original_pic_for_transfer_agreement_2 = self.pic_for_transfer_agreement_2
        # self.__original_pic_for_position_change = self.pic_for_position_change
        # self.__original_pic_for_quit = self.pic_for_quit

    # def upload_image_path(self, filename, contract_type=None, *args, **kwargs):
    #     isThumb = ''
    #     if 'thumb' in contract_type:
    #         isThumb = 'thumb_'
    #     new_filename = isThumb + str(random.randint(1,3910209312))
    #     name, ext = get_filename_ext(filename)
    #     final_filename = '{new_filename}{ext}'.format(new_filename=new_filename, ext=ext)
    #     return 'image/' + imageWithModelName + "/{sales_id}/{contract_type}/{final_filename}".format(
	# 			sales_id=self.sales_id,
	#             contract_type=contract_type.split('.')[0],
	# 			final_filename=final_filename
	# 			)

    # def saveThumb(self, self_origin, self_thumb):
    #     size = 145, 205
    #     im = Image.open(getattr(self, self_origin))
    #     origin_image_io = BytesIO()
    #     im.save(origin_image_io, format=im.format)
    #     getattr(self, self_origin).save(
	# 			Sales.upload_image_path(self, getattr(self, self_origin).name.split('/')[-1], getattr(self, self_origin).field.contract_type),
	# 			content=ContentFile(origin_image_io.getvalue()),
	# 			save=False
	# 	)
    #     width = im.width
    #     height = im.height
	#
    #     if ((width/height)*205) > 145:
    #         newWidth = 145
    #         newHeight = 145 * (height/width)
    #     else:
    #         newHeight = 205
    #         newWidth  = (width/height) * 205
	#
    #     new_image_io = BytesIO()
    #     new_image = im.resize(((int)(newWidth), (int)(newHeight)), Image.ANTIALIAS)
    #     new_image.save(new_image_io, format=im.format)
    #     getattr(self, self_thumb).save(
	# 			Sales.upload_image_path(self, getattr(self, self_origin).name.split('/')[-1], getattr(self, self_thumb).field.contract_type),
	# 			content=ContentFile(new_image_io.getvalue()),
	# 			save=False
	# 	)


    # def save(self, *args, **kwargs):
    #     argsDict = (
	# 		('pic_for_contract_1', self.__original_pic_for_contract_1, 'pic_for_contract_1_thumb', 'pic_for_contract_1_datetime'),
	# 		('pic_for_contract_2', self.__original_pic_for_contract_2, 'pic_for_contract_2_thumb', 'pic_for_contract_2_datetime'),
	# 		('pic_for_contract_3', self.__original_pic_for_contract_3, 'pic_for_contract_3_thumb', 'pic_for_contract_3_datetime'),
	# 		('pic_for_contract_4', self.__original_pic_for_contract_4, 'pic_for_contract_4_thumb', 'pic_for_contract_4_datetime'),
	# 		('pic_for_contract_5', self.__original_pic_for_contract_5, 'pic_for_contract_5_thumb', 'pic_for_contract_5_datetime'),
	#
	# 		('pic_for_basic_data_1', self.__original_pic_for_basic_data_1, 'pic_for_basic_data_1_thumb', 'pic_for_basic_data_1_datetime'),
	# 		('pic_for_basic_data_2', self.__original_pic_for_basic_data_2, 'pic_for_basic_data_2_thumb', 'pic_for_basic_data_2_datetime'),
	# 		('pic_for_basic_data_3', self.__original_pic_for_basic_data_3, 'pic_for_basic_data_3_thumb', 'pic_for_basic_data_3_datetime'),
	#
	# 		('pic_for_agent_agreement_1', self.__original_pic_for_agent_agreement_1, 'pic_for_agent_agreement_1_thumb', 'pic_for_agent_agreement_1_datetime'),
	# 		('pic_for_agent_agreement_2', self.__original_pic_for_agent_agreement_2, 'pic_for_agent_agreement_2_thumb', 'pic_for_agent_agreement_2_datetime'),
	#
	# 		('pic_for_transfer_agreement_1', self.__original_pic_for_transfer_agreement_1, 'pic_for_transfer_agreement_1_thumb', 'pic_for_transfer_agreement_1_datetime'),
	# 		('pic_for_transfer_agreement_2', self.__original_pic_for_transfer_agreement_2, 'pic_for_transfer_agreement_2_thumb', 'pic_for_transfer_agreement_2_datetime'),
	#
	# 		('pic_for_position_change', self.__original_pic_for_position_change, 'pic_for_position_change_thumb', 'pic_for_position_change_datetime'),
	# 		('pic_for_quit', self.__original_pic_for_quit, 'pic_for_quit_thumb', 'pic_for_quit_datetime'),
	#
	# 	 )
    #     for i in argsDict:
    #         if getattr(self, i[0]) != i[1]:
    #             try:
    #                 Sales.saveThumb(self, i[0], i[2])
    #                 setattr(self, i[3], timezone.now())
    #             except:
    #                 setattr(self, i[0], None)
    #                 setattr(self, i[3], None)
    #                 setattr(self, i[2], None)
    #     super(Sales, self).save(*args, **kwargs)
    def __str__(self):
        return self.full_name
