from rest_framework import serializers
from django.utils.timezone import now
from django.db.transaction import atomic
from .models import Sales


class DynamicFieldsModelSerializer(serializers.ModelSerializer):
    """
    A ModelSerializer that takes an additional `fields` argument that
    controls which fields should be displayed.
    """

    def __init__(self, *args, **kwargs):
        # Instantiate the superclass normally
        super(DynamicFieldsModelSerializer, self).__init__(*args, **kwargs)

        fields = self.context['request'].query_params.get('fields')
        if fields:
            fields = fields.split(',')
            # Drop any fields that are not specified in the `fields` argument.
            allowed = set(fields)
            existing = set(self.fields.keys())
            for field_name in existing - allowed:
                self.fields.pop(field_name)
        else:
            fields = '__all__'


class SalesSerializer(DynamicFieldsModelSerializer):
    class Meta:
        model = Sales
        fields = '__all__'



#class SalesSerializer(serializers.ModelSerializer):
#
#    class Meta:
#        model = Sales
#        fields = '__all__'

class SalesSerializerForUpdateMemo(serializers.ModelSerializer):

    class Meta:
        model = Sales
        fields = ('memo', 'id')

class SalesBankInfoSerializer(serializers.ModelSerializer):

    class Meta:
        model = Sales
        fields = ('bank_num', 'bank_pri_name', 'bank_sub_name', 'id', 'full_name', 'bank_account')

class SalesForSelectSerializer(serializers.ModelSerializer):
    company__company_name = serializers.CharField(max_length=200)
    position__title = serializers.CharField(max_length=200)
    sales_unit__display_name = serializers.CharField(max_length=200)
    class Meta:
        model = Sales
        fields = ('sales_id', 'sales_unit', 'company', 'id', 'state', 'cellphone_one', 'full_name', 'position', 'work_type', 'sales_unit__display_name', 'position__title', 'company__company_name')

class SalesUnitAllSerializer(serializers.ModelSerializer):
    company__company_name = serializers.CharField(max_length=200)
    position__title = serializers.CharField(max_length=200)
    position__rank = serializers.DecimalField(max_digits=5, decimal_places=0, source='position.rank')
    sales_unit__display_name = serializers.CharField(max_length=200)
    class Meta:
        model = Sales
        fields = (
            'sales_id',
            'full_name',
            'sales_unit',
            'id',
            'company',
            'position',
            'work_type',
            'company__company_name',
            'position__title',
            'position__rank',
            'sales_unit__display_name')

class SalesUnitAllSerializerIntoLeave(serializers.ModelSerializer):
    company__company_name = serializers.CharField(max_length=200)
    position__title = serializers.CharField(max_length=200)
    sales_unit__display_name = serializers.CharField(max_length=200)
    into_leave = serializers.BooleanField()
    class Meta:
        model = Sales
        fields = (
            'sales_id',
            'full_name',
            'sales_unit',
            'id',
            'company',
            'position',
            'work_type',
            'company__company_name',
            'position__title',
            'sales_unit__display_name',
            'into_leave')
    # def create(self, validated_data):
    #     print(validated_data)
    #     test = Test(
    #             created = validated_data.get('created', None),
    #             user = validated_data.get('user', None),
    #             title = validated_data.get('title', None),
    #             img1 = validated_data.get('img1', None),
    #             img1_thumb = validated_data.get('img1_thumb', None),
    #             img1_datetime = validated_data.get('img1_datetime', None),
    #             img2 = validated_data.get('img2', None),
    #             img2_datetime = validated_data.get('img2_datetime', None),
    #         )
    #     test.save(force_insert=True ,force_update=True, api='Create')
    #     return test
    #
    # def update(self, instance, validated_data):
    #     print(validated_data)
    #     instance.user = validated_data.get('user', instance.user)
    #     instance.title = validated_data.get('title', instance.title)
    #     instance.img1 = validated_data.get('img1', instance.img1)
    #     instance.img1_thumb = validated_data.get('img1_thumb', instance.img1_thumb)
    #     instance.img1_datetime = validated_data.get('img1_datetime', instance.img1_datetime)
    #     instance.img2 = validated_data.get('img2', instance.img2)
    #     instance.img2_datetime = validated_data.get('img2_datetime', instance.img2_datetime)
    #     instance.save(force_insert=False ,force_update=True, api='Update')
    #     return instance
