from django.shortcuts import render, redirect
from .serializers import SalesSerializer, SalesBankInfoSerializer, SalesUnitAllSerializer, SalesForSelectSerializer, SalesSerializerForUpdateMemo, SalesUnitAllSerializerIntoLeave
from .models import Sales

from django.contrib.auth.decorators import login_required
from accounts.models import User
from salesUnit.models import SalesUnit
from position.models import Position
from company.models import Company

from rest_framework import generics, viewsets
from rest_framework.permissions import IsAuthenticated
# Create your views here.
class SalesViewSet(viewsets.ModelViewSet):
    queryset = Sales.objects.all()
    serializer_class = SalesSerializer
    permission_classes = (IsAuthenticated,)

class SalesUpdateMemo(generics.UpdateAPIView):
    serializer_class = SalesSerializerForUpdateMemo
    permission_classes = (IsAuthenticated,)
    queryset = Sales.objects.all()
    def put(self, request, *args, **kwargs):
        return self.update(request, *args, **kwargs)

class SalesListForSelect(generics.ListAPIView):
    serializer_class = SalesForSelectSerializer
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        user = self.request.user
        if user:
            queryset = Sales.objects.filter(sales_unit__in = user.unit.all())
            query = self.request.GET.get("query", None)
            if not query:
                queryset = queryset.select_related()
                unit = []
                position = []
                try:
                    for i in self.request.GET.get("unit", None).split(","):
                        unit.append((int)(i))
                        # print(i)
                    for i in self.request.GET.get("position", None).split(","):
                        position.append(i)
                        # print(i)
                except:
                    pass
                work_type = self.request.GET.get("work_type", None)
                state = self.request.GET.get("state", None)
                queryset = queryset.filter(sales_unit__id__in = unit)
                queryset = queryset.filter(position__id__in = position)
                if work_type == "業務" or  work_type == "電訪":
                    queryset = queryset.filter(work_type=work_type)
                if state == "在職" or state == "離職":
                    queryset = queryset.filter(state=state)
                output = []
                queryset = queryset.order_by('sales_unit', 'sales_id')
                for i in queryset.values(
                    "sales_id",
                    "full_name",
                    "id",
                    "sales_unit",
                    "company",
                    "position",
                    "work_type",
                    "cellphone_one",
                    "company__company_name",
                    "position__title",
                    "sales_unit__display_name",
                    "state"):
                    output.append(
                        {
                            "sales_id" : i["sales_id"],
                            "full_name" : i["full_name"],
                            "id" : i["id"],
                            "work_type" : i["work_type"],
                            "sales_unit": SalesUnit.objects.filter(id=i["sales_unit"]).first(),
                            "company" : Company.objects.filter(id=i["company"]).first(),
                            "position" : Position.objects.filter(id=i["position"]).first(),
                            "cellphone_one": i["cellphone_one"],
                            "company__company_name" : i["company__company_name"],
                            "position__title" : i["position__title"],
                            "sales_unit__display_name" : i["sales_unit__display_name"],
                            "state": i["state"]
                        }
                    )
                return output
            else:
                cellphone_one = queryset.filter(cellphone_one=query)
                cellphone_two = queryset.filter(cellphone_two=query)
                full_name = queryset.filter(full_name=query)
                queryset = cellphone_one | cellphone_two | full_name
                output = []
                for i in queryset.values(
                    "sales_id",
                    "full_name",
                    "id",
                    "sales_unit",
                    "company",
                    "position",
                    "work_type",
                    "cellphone_one",
                    "company__company_name",
                    "position__title",
                    "sales_unit__display_name",
                    "state"):
                    output.append(
                        {
                            "sales_id" : i["sales_id"],
                            "full_name" : i["full_name"],
                            "id" : i["id"],
                            "work_type" : i["work_type"],
                            "sales_unit": SalesUnit.objects.filter(id=i["sales_unit"]).first(),
                            "company" : Company.objects.filter(id=i["company"]).first(),
                            "position" : Position.objects.filter(id=i["position"]).first(),
                            "cellphone_one": i["cellphone_one"],
                            "company__company_name" : i["company__company_name"],
                            "position__title" : i["position__title"],
                            "sales_unit__display_name" : i["sales_unit__display_name"],
                            "state": i["state"]
                        }
                    )
                return output
        return []

class SalesListByUnitForIm(generics.ListAPIView):
    serializer_class = SalesUnitAllSerializer
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        """
        Optionally restricts the returned purchases to a given user,
        by filtering against a `username` query parameter in the URL.
        """
        queryset = Sales.objects.all()
        queryset = queryset.select_related()
        gte = queryset.filter(state="在職", position__rank__gte=17)
        query_salesUnit = self.kwargs.get('salesUnit', None)
        if (query_salesUnit is not None) or (type(query_salesUnit) != type(1)):
            queryset = queryset.filter(sales_unit=query_salesUnit, state="在職")
            queryset = queryset | gte
            queryset = queryset.order_by('sales_id')
            output = []
            for i in queryset.values(
                "sales_id",
                "full_name",
                "id",
                "sales_unit",
                "company",
                "position",
                "work_type",
                "company__company_name",
                "position__title",
                "position__rank",
                "sales_unit__display_name"):
                output.append(
                    {
                        "sales_id" : i["sales_id"],
                        "full_name" : i["full_name"],
                        "id" : i["id"],
                        "work_type" : i["work_type"],
                        "sales_unit": SalesUnit.objects.filter(id=i["sales_unit"]).first(),
                        "company" : Company.objects.filter(id=i["company"]).first(),
                        "position" : Position.objects.filter(id=i["position"]).first(),
                        "company__company_name" : i["company__company_name"],
                        "position_rank": Position.objects.filter(id=i["position"]).first().rank,
                        "position__title" : i["position__title"],
                        "sales_unit__display_name" : i["sales_unit__display_name"]
                    }
                )
            return output
        else:
            return []

class SalesListByUnit(generics.ListAPIView):
    serializer_class = SalesUnitAllSerializerIntoLeave
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        """
        Optionally restricts the returned purchases to a given user,
        by filtering against a `username` query parameter in the URL.
        """
        queryset = Sales.objects.all()
        queryset = queryset.select_related()
        query_salesUnit = self.kwargs.get('salesUnit', None)
        if (query_salesUnit is not None) or (type(query_salesUnit) != type(1)):
            queryset = queryset.filter(sales_unit=query_salesUnit, state="在職",)
            queryset = queryset.order_by('sales_id')
            output = []
            for i in queryset.values(
                "sales_id",
                "full_name",
                "id",
                "sales_unit",
                "company",
                "position",
                "work_type",
                "company__company_name",
                "position__title",
                "sales_unit__display_name"):
                this_sales = Sales.objects.get(id=i['id'])
                into_leave = False
                if this_sales.into_whos_back_account:
                    if "Sales " in this_sales.into_whos_back_account:
                        this_head = Sales.objects.get(id=this_sales.into_whos_back_account.split("Sales ")[-1])
                        if this_head.state != "在職":
                            into_leave = True
                output.append(
                    {
                        "sales_id" : i["sales_id"],
                        "full_name" : i["full_name"],
                        "id" : i["id"],
                        "work_type" : i["work_type"],
                        "sales_unit": SalesUnit.objects.filter(id=i["sales_unit"]).first(),
                        "company" : Company.objects.filter(id=i["company"]).first(),
                        "position" : Position.objects.filter(id=i["position"]).first(),
                        "company__company_name" : i["company__company_name"],
                        "position__title" : i["position__title"],
                        "sales_unit__display_name" : i["sales_unit__display_name"],
                        "into_leave": into_leave
                    }
                )
            return output
        else:
            return []

class SalesListByUnitIS(generics.ListAPIView):
    serializer_class = SalesUnitAllSerializer
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        """
        Optionally restricts the returned purchases to a given user,
        by filtering against a `username` query parameter in the URL.
        """
        queryset = Sales.objects.all()
        queryset = queryset.select_related()
        query_salesUnit = self.kwargs.get('salesUnit', None)
        if (query_salesUnit is not None) or (type(query_salesUnit) != type(1)):
            queryset = queryset.filter(sales_unit=query_salesUnit, state="在職", isHuman=True)
            queryset = queryset.order_by('sales_id')
            output = []
            for i in queryset.values(
                "sales_id",
                "full_name",
                "id",
                "sales_unit",
                "company",
                "position",
                "work_type",
                "company__company_name",
                "position__title",
                "sales_unit__display_name"):
                output.append(
                    {
                        "sales_id" : i["sales_id"],
                        "full_name" : i["full_name"],
                        "id" : i["id"],
                        "work_type" : i["work_type"],
                        "sales_unit": SalesUnit.objects.filter(id=i["sales_unit"]).first(),
                        "company" : Company.objects.filter(id=i["company"]).first(),
                        "position" : Position.objects.filter(id=i["position"]).first(),
                        "company__company_name" : i["company__company_name"],
                        "position__title" : i["position__title"],
                        "sales_unit__display_name" : i["sales_unit__display_name"]
                    }
                )
            return output
        else:
            return []

class SalesBankInfo(generics.ListAPIView):
    serializer_class = SalesBankInfoSerializer
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        """
        Optionally restricts the returned purchases to a given user,
        by filtering against a `username` query parameter in the URL.
        """
        queryset = Sales.objects.all()
        query_id = self.kwargs.get('id', None)
        if (query_id is not None) or (type(query_id) != type(1)):
            queryset = queryset.filter(id=query_id)
        queryset = queryset.order_by('sales_id')
        return queryset


@login_required(login_url='/login/')
def searchSales(request):
    if request.user.shenfen != "catchdog":
        return redirect("/main/")
    sales_unit = SalesUnit.objects.all()
    content = {
        	"units": sales_unit,
        }
    if request.method == 'GET':
        query = request.GET.get("query", None)
        unit_id  = request.GET.get("sales_unit", None)
        if query:
            full_name = Sales.objects.filter(full_name__contains=query)
            cellphone_one = Sales.objects.filter(cellphone_one__contains=query)
            result = full_name | cellphone_one
            if unit_id == "all":
                result = result.filter(state="在職").order_by("sales_unit")
            else:
                result = result.filter(state="在職").filter(sales_unit__id=unit_id).order_by("sales_unit")
            content = {
                "result": result,
                "units": sales_unit,
            }

    return render(request, "threefloor.html", content)
# Create your views here.
