"""app URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.11/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
from django.conf.urls import url, include
from django.contrib import admin
from django.conf.urls.static import static
from django.conf import settings
from idiomSearch.views import idiomSearch
from rest_framework.routers import DefaultRouter
from testapp.views import TestViewSet
from company.views import CompanyViewSet, CompanyList
from sales.views import SalesListByUnit, SalesViewSet, SalesBankInfo, SalesListForSelect, SalesUpdateMemo, SalesListByUnitIS, SalesListByUnitForIm, searchSales
from tempSales.views import TempSalesViewSet, TempSalesBasicInfoView, TempSalesCUset, TempSalesCreatView
from application.views import ApplicationListAPI, ApplicationViewSet
from notification.views import NotificationViewSet, NotificationListAPI
from modified.views import ModifiedListAPI, ModifiedViewSet
from .views import home_page, about_page, contact_page, login_page, hr_page, sales_page, main_page, logout_func
from accounts.views import change_password_page, user_login, list_sales_by_unit, get_sales_data
#from coltest.views import ColtestListView, coltest_list_view, ColtestCreate, Coltest_detail_view, ColtestDetailView, Coltest_update_view
from testapp.views import ColtestListView
router = DefaultRouter()
router.register(r'test', TestViewSet)
router.register(r'company', CompanyViewSet)
router.register(r'sales', SalesViewSet)
router.register(r'tempSales', TempSalesCUset)
router.register(r'application', ApplicationViewSet)
router.register(r'notification', NotificationViewSet)
router.register(r'modified', ModifiedViewSet)
# router.register(r'application', ApplicationListAPI)

urlpatterns = [
    url(r'^$', home_page),
    url(r'^sales/', sales_page),
    url(r'^hr/', hr_page),
    url(r'^login/', login_page),
    url(r'^about/', about_page),
    url(r'^main/', main_page),
    url(r'^logout/', logout_func),
    url(r'api/', include(router.urls)),
    url(r'api/notification_list/', NotificationListAPI.as_view()),
    url(r'api/tempSales/list/(?P<id>\d+)/$',TempSalesViewSet.as_view()),
    url(r'api/company/list/(?P<salesUnit>\d+)/$', CompanyList.as_view()),
    url(r'api/sales/listByUnitIm/(?P<salesUnit>\d+)/$', SalesListByUnitForIm.as_view()),
    url(r'api/sales/listByUnit/(?P<salesUnit>\d+)/$', SalesListByUnit.as_view()),
    url(r'api/sales/listByUnitIS/(?P<salesUnit>\d+)/$', SalesListByUnitIS.as_view()),
    url(r'api/sales/bankInfo/(?P<id>\d+)/$', SalesBankInfo.as_view()),
    url(r'api/listForSelect/', SalesListForSelect.as_view()),
    url(r'api/salesUpdateMemo/(?P<pk>\d+)/$', SalesUpdateMemo.as_view()),
    url(r'api/tempSales/basicInfo/(?P<tempSales>\d+)/$', TempSalesBasicInfoView.as_view()),
    url(r'api/updateSales/', TempSalesCreatView.as_view()),
    url(r'api/applicationList/', ApplicationListAPI.as_view()),
    url(r'api/modifiedList/', ModifiedListAPI.as_view()),
    url(r'api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    url(r'api/user_login/', user_login),
    url(r'api/get_sales_data/', get_sales_data),
    url(r'api/list_sales_by_unit/', list_sales_by_unit),
    # url(r'^coltest-detail/(?P<pk>\d+)/$', ColtestDetailView.as_view()),
    # url(r'^coltest-detail-fbv/(?P<pk>\d+)/$', Coltest_detail_view),
    # url(r'^coltest-update/(?P<pk>\d+)/$', Coltest_update_view),
    # url(r'^coltest-create/', ColtestCreate.as_view()),
    # url(r'^echo/(?P<userid>[0-9]+)$', echo, name='echo'),
    url(r'^coltest/', ColtestListView.as_view()),
    url(r'^change_password/', change_password_page),
    # url(r'^coltest-fbv/', coltest_list_view),
    url(r'^contact/', contact_page),
    url(r'^searchSales/', searchSales),
    url(r'^admin/', admin.site.urls),
    url(r'^idiomSearch/', idiomSearch),
]

if settings.DEBUG:
    urlpatterns = urlpatterns + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
    urlpatterns = urlpatterns + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
