from django.shortcuts import render, redirect
from django.template import Library
from django.http import HttpResponse
from .forms import ContactForm, LoginForm
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from accounts.models import User
from django.core.exceptions import PermissionDenied
from position.models import Position
from company.models import Company
import pytz
from datetime import datetime, timedelta

TaipeiTz = pytz.timezone('Asia/Taipei')

def check_update_password(func, request):
    def with_check(*args, **kwargs):
        user = request.user
        if user.profile.updated.astimezone(TaipeiTz) < TaipeiTz.localize(datetime.today()) - timedelta(days=90):
			return redirect("/change_password/")
		else:
			return func(*args, **kwargs)
    return with_check

@login_required(login_url='/login/')
def home_page(request):
	content = {
		"title": "this is Home page"
	}
	return render(request, "home_page.html", content)

@login_required(login_url='/login/')
def logout_func(request):
	logout(request)
	return redirect("/login/")

# class RequestUserFilter(logging.Filter):
#     def filter(self, record):
#         record.user = record.request.user
#         return True

@login_required(login_url='/login/')
def main_page(request):
	user = {
		"user": request.user,
		"position": Position.objects.all().order_by("rank"),
		"company": Company.objects.all()
	}
	return render(request, "main_page.html", user)

@login_required(login_url='/login/')
def about_page(request):
	content = {
		"title": "this is About page"
	}
	return render(request, "about_page.html", content)

@login_required(login_url='/login/')
def contact_page(request):
	contact_form = ContactForm(request.POST or None)
	content = {
		"title": "this is Contact page",
		"content": "",
		"form": contact_form
	}
	# if contact_form.is_valid():
		# print(contact_form.cleaned_data)
	# if request.method == "POST":
	# 	print(request.POST.get('email'))
	# 	print(request.POST.get('email'))
	# 	print(request.POST.get('email'))

	return render(request, "contact_page.html", content)

def login_page(request):
	if request.user.is_authenticated():
		return redirect("/")
	form = LoginForm(request.POST or None)
	# print(request.user.is_authenticated())
	content = {
		"form": form,
		"error":""
	}


	if form.is_valid():
		username = form.cleaned_data.get("username")
		password = form.cleaned_data.get("password")
		user = authenticate(request, username=username, password=password)
		if user is not None:
			user_obj = User.objects.get(email=username)
			login(request, user)
			if user_obj.profile.updated.astimezone(TaipeiTz) < TaipeiTz.localize(datetime.today()) - timedelta(days=90):
				return redirect("/change_password/")
			if not user_obj.is_active:
				raise Exception("this account have no permission to access this service")
			elif user_obj.is_admin:
				return redirect("/main/")
			else:
				if user_obj.get_shenfen() == 'salesAssistant':
					return redirect("/main/")
				elif user_obj.get_shenfen() == 'multiUnitSalesAssistant':
					return redirect("/main/")
				elif user_obj.get_shenfen() == 'salesHumanResources':
					return redirect("/main/")
				elif user_obj.get_shenfen() == 'humanResources':
					return redirect("/main/")
		else:
			content["error"] = "錯誤的信箱或密碼"
	return render(request, "login_page.html", content)

@login_required(login_url='/login/')
def hr_page(request):
	content = {"title":'hr_page'}
	return render(request, "hr_page.html", content)

@login_required(login_url='/login/')
def sales_page(request):
	username = request.user.email
	user_obj = User.objects.get(email=username)
	shenfen = user_obj.get_shenfen()
	if shenfen == 'salesAssistant' :
		content = {"title":'sales_page'}
	elif shenfen == 'multiUnitSalesAssistant':
		content = {"title":'sales_page'}
	elif shenfen == 'salesHumanResources':
		contetn = {"title":'sales_page'}
	elif shenfen == 'humanResources':
		contetn = {"title":'sales_page'}
	else:
		return redirect("/")
	return render(request, "sales_page.html", content)
