# Create your views here.
from django.core.files.uploadedfile import SimpleUploadedFile, InMemoryUploadedFile
from django.core.files.base import ContentFile
from django.utils import timezone

from .models import TempSales
from .serializers import TempSalesSerializer, TempSalesBasicInfoSerializer, TempSalesViewSerializer
from sales.models import Sales
from application.models import Application


from rest_framework import status
from rest_framework.response import Response
from rest_framework import serializers
from rest_framework import viewsets, generics
from rest_framework.permissions import IsAuthenticated

from datetime import timedelta, datetime
from PIL import Image
import os, random
from io import BytesIO
import base64
import string

def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))

# Create your views here.
class TempSalesBasicInfoView(generics.ListAPIView):
    serializer_class = TempSalesBasicInfoSerializer
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        """
        Optionally restricts the returned purchases to a given user,
        by filtering against a `username` query parameter in the URL.
        """
        queryset = TempSales.objects.all()
        query_id = self.kwargs.get('tempSales', None)
        if (query_id is not None) or (type(query_id) != type(1)):
            queryset = queryset.filter(id=query_id)
        return queryset
class TempSalesCreatView(generics.CreateAPIView):
    queryset = TempSales.objects.all()
    serializer_class = TempSalesSerializer
    permission_classes = (IsAuthenticated,)
    def create(self, request, *args, **kwargs):
        old_sales_roc_id = request.POST.get('old_sales', None)
        # print(old_sales_roc_id, request.data['sales_unit'])
        if Application.objects.filter(use="修改資料", tempSales__roc_id=old_sales_roc_id, unit=request.data['sales_unit']).exclude(status="申請完成").exclude(status="刪除"):
            # print(old_sales_roc_id, request.data['sales_unit'])
            raise serializers.ValidationError('該員工資料修改已在申請中')
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

class TempSalesViewSet(generics.ListAPIView):
    serializer_class = TempSalesViewSerializer
    permission_classes = (IsAuthenticated,)
    def get_queryset(self):
        """
        Optionally restricts the returned purchases to a given user,
        by filtering against a `username` query parameter in the URL.
        """
        queryset = TempSales.objects.all()
        query_id = self.kwargs.get('id', None)
        if (query_id is not None) or (type(query_id) != type(1)):
            queryset = queryset.filter(id=query_id)
        return queryset

class TempSalesCUset(viewsets.ModelViewSet):
    queryset = TempSales.objects.all()
    serializer_class = TempSalesSerializer
    permission_classes = (IsAuthenticated,)
    def get_serializer_context(self):
        return {
            'request': self.request, # request object is passed here
            'format': self.format_kwarg,
            'view': self
        }
    def partial_update(self, instance, validated_data):
        pass
    def create(self, request,):
        if request.POST.get('old_sales', None) == None:
            # argsDict = (
            #              'pic_for_contract_1', 'pic_for_contract_2', 'pic_for_contract_3', 'pic_for_contract_4',
            #              'pic_for_contract_5', 'pic_for_basic_data_1', 'pic_for_basic_data_2', 'pic_for_basic_data_3',
            #              'pic_for_agent_agreement_1', 'pic_for_agent_agreement_2', 'pic_for_transfer_agreement_1',
            #              'pic_for_transfer_agreement_2', 'pic_for_position_change', 'pic_for_quit'
            #            )
            mutable = request.data._mutable
            request.data._mutable = True
            #
            roc_sales = Sales.objects.filter(roc_id=request.data['roc_id'])
            if roc_sales:
                for i in roc_sales:
                    if i.state == "在職":
                        raise serializers.ValidationError('重複的身分證字號')
                    else:
                        if not (i.date_of_leave < (datetime.now().date() - timedelta(days=6*30))) :
                            if (str)(i.sales_unit_id) != (str)(request.data['sales_unit']):
                                raise serializers.ValidationError('離職後半年欲復職需在同單位')
            if Application.objects.filter(use="新增業務", tempSales__roc_id=request.data['roc_id'], unit=request.data['sales_unit']).exclude(status="申請完成").exclude(status="刪除"):
                raise serializers.ValidationError('該身分證已在申請中')

            # for i in argsDict:
            #     if request.data.get(i, '') != '':
            #         if 'base64' in request.data[i] :
            #             im = Image.open(BytesIO(base64.b64decode(request.data[i].split(';base64,')[1])))
            #         else:
            #             im = Image.open(request.data[i])
            #         origin_image_io = BytesIO()
            #         im.save(origin_image_io, format=im.format)
            #         request.data[i] = SimpleUploadedFile(
            #                                 id_generator()+'.png',
            #                                 origin_image_io.getvalue(),
            #                                 'image/jpeg',
            #                             )
            #     else:
            #         request.data.pop(i)
            request.data['sales_id'] = id_generator()
            if request.data['into_whos_back_account'] != None:
                request.data['into_whos_back_account'] = request.data['into_whos_back_account']
            else:
                request.data.pop('into_whos_back_account')

            if request.data['bank_account_name_2'] != None:
                request.data['bank_account_name_2'] = request.data['bank_account_name_2']
            else:
                request.data.pop('bank_account_name_2')

            if request.data['bank_account_name_3'] != None:
                request.data['bank_account_name_3'] = request.data['bank_account_name_3']
            else:
                request.data.pop('bank_account_name_3')

            if request.data['company'] != "不選擇":
                try:
                    if type(request.data['company']) != type(1):
                        request.data['company'] = (int)(request.data['company'])
                except(ValueError):
                    request.data.pop('company')
            else:
                request.data.pop('company')
            try:
                if type(request.data['immediate_supervisor']) != type(1):
                    request.data['immediate_supervisor'] = (int)(request.data['immediate_supervisor'])
            except(ValueError):
                request.data.pop('immediate_supervisor')
            request.data._mutable = mutable
            return super().create(request)
        else:
            return super().create(request)
    def update(self, request, pk=None):
        # argsDict = (
        #              'pic_for_contract_1', 'pic_for_contract_2', 'pic_for_contract_3', 'pic_for_contract_4',
        #              'pic_for_contract_5', 'pic_for_basic_data_1', 'pic_for_basic_data_2', 'pic_for_basic_data_3',
        #              'pic_for_agent_agreement_1', 'pic_for_agent_agreement_2', 'pic_for_transfer_agreement_1',
        #              'pic_for_transfer_agreement_2', 'pic_for_position_change', 'pic_for_quit'
        #            )
        mutable = request.data._mutable
        request.data._mutable = True
        print(request.data['pdfupload'])
        if request.data['application'] == "deliver":
            if Application.objects.filter(id=request.data['app'])[0].use == "修改資料":
                if Sales.objects.filter(sales_unit=request.data['sales_unit'], roc_id=request.data['roc_id']):
                    if Application.objects.filter(id=request.data['app'])[0].origin_user != Sales.objects.filter(sales_unit=request.data['sales_unit'], roc_id=request.data['roc_id'])[0]:
                        raise serializers.ValidationError('重複的身分證字號')
                elif Application.objects.filter(use="修改資料", tempSales__roc_id=request.data['roc_id'], unit=request.data['sales_unit']).exclude(status="申請完成").exclude(status="刪除"):
                    raise serializers.ValidationError('該身分證已在申請中')
            else:
                if Sales.objects.filter(sales_unit=request.data['sales_unit'], roc_id=request.data['roc_id'], state="在職"):
                    raise serializers.ValidationError('重複的身分證字號')
                elif Application.objects.filter(use="新增業務", tempSales__roc_id=request.data['roc_id'], unit=request.data['sales_unit']).exclude(status="申請完成").exclude(status="刪除").exclude(id=Application.objects.filter(id=request.data['app'])[0].id):
                    raise serializers.ValidationError('該身分證已在申請中')
        # for i in argsDict:
        #     if request.data[i] != '' and request.data[i] != "maintain":
        #         if 'base64' in request.data[i] :
        #             im = Image.open(BytesIO(base64.b64decode(request.data[i].split(';base64,')[1])))
        #         else:
        #             im = Image.open(request.data[i])
        #         origin_image_io = BytesIO()
        #         im.save(origin_image_io, format=im.format)
        #         request.data[i] = SimpleUploadedFile(
        #                                 id_generator()+'.png',
        #                                 origin_image_io.getvalue(),
        #                                 'image/jpeg',
        #                             )
        #     elif request.data[i] == "maintain":
        #         request.data.pop(i)
        if request.data['pdfupload'] =="maintain":
            request.data.pop('pdfupload')
        elif request.data['pdfupload'] =="undefined":
            request.data.pop('pdfupload')
        request.data['sales_id'] = id_generator()
        # if request.data['into_whos_back_account'] != "本人":
        #     try:
        #         if type(request.data['into_whos_back_account']) != type(1):
        #             request.data['into_whos_back_account'] = (int)(request.data['into_whos_back_account'])
        #     except(ValueError):
        #         request.data.pop('into_whos_back_account')
        # else:
        #     request.data.pop('into_whos_back_account')
        if request.data['into_whos_back_account'] != None:
            request.data['into_whos_back_account'] = request.data['into_whos_back_account']
        else:
            request.data.pop('into_whos_back_account')

        if request.data['bank_account_name_2'] != None:
            request.data['bank_account_name_2'] = request.data['bank_account_name_2']
        else:
            request.data.pop('bank_account_name_2')

        if request.data['bank_account_name_3'] != None:
            request.data['bank_account_name_3'] = request.data['bank_account_name_3']
        else:
            request.data.pop('bank_account_name_3')

        if request.data['company'] != "不選擇":
            try:
                if type(request.data['company']) != type(1):
                    request.data['company'] = (int)(request.data['company'])
            except(ValueError):
                request.data.pop('company')
        else:
            request.data['company'] = None
        try:
            if type(request.data['immediate_supervisor']) != type(1):
                request.data['immediate_supervisor'] = (int)(request.data['immediate_supervisor'])
        except(ValueError):
            request.data.pop('immediate_supervisor')
        request.data._mutable = mutable
        return super().update(request)
