from rest_framework import serializers

from django.utils.timezone import now
from django.db.transaction import atomic

from .models import TempSales

from application.models import Application
from accounts.models import User
from sales.models import Sales
from log.models import LogForUserBehavior

from datetime import datetime
from datetime import timedelta
import pytz

class TempSalesSerializer(serializers.ModelSerializer):
    # application = serializers.CharField(max_length=200)
    class Meta:
        model = TempSales
        fields = '__all__'

    @atomic
    def create(self, validated_data):
        user_id = self.context['request'].user.id
        if self.context['request'].data.get('old_sales', None) == None:
            tempSales = TempSales()
            application = Application()
            logForUserBehavior = LogForUserBehavior()
            for i in validated_data:
                setattr(tempSales, i, validated_data[i])

            tempSales.save()

            if self.context['request'].data['application'] == "save":
                setattr(application, 'status', '編輯中')
                setattr(logForUserBehavior, 'behavior', "新增業務-編輯中")
            elif self.context['request'].data['application'] == "deliver":
                setattr(application, 'status', '待審核')
                setattr(logForUserBehavior, 'behavior', "新增業務-待審核")
                setattr(tempSales, 'date_of_come', datetime.now(pytz.timezone('Asia/Taipei')).date())
                tempSales.save()

            setattr(application, 'user', User.objects.get(id=user_id))
            setattr(application, 'unit', validated_data['sales_unit'])
            setattr(application, 'use', '新增業務')
            setattr(application, 'tempSales', tempSales)
            application.save()

            setattr(logForUserBehavior, 'application', application)
            setattr(logForUserBehavior, 'full_name', User.objects.get(id=user_id).full_name)
            setattr(logForUserBehavior, 'user', User.objects.get(id=user_id))
            logForUserBehavior.save()
            return tempSales
        else:
            tempSales = TempSales()
            application = Application()
            logForUserBehavior = LogForUserBehavior()
            sales = Sales.objects.filter(roc_id=self.context['request'].data.get('old_sales', None)).first()
            for i in sales.__dict__:
                if (i != 'id') and (i !='_state') and ('Sales' not in i):
                    setattr(tempSales, i, getattr(sales,i))
            setattr(tempSales, 'sales_id', getattr(sales, 'sales_id'))
            tempSales.save()
            setattr(application, 'status', '編輯中')
            setattr(logForUserBehavior, 'behavior', "修改資料-編輯中")
            setattr(application, 'origin_user', sales)
            setattr(application, 'user', User.objects.get(id=user_id))
            setattr(application, 'unit', sales.sales_unit)
            setattr(application, 'use', '修改資料')
            setattr(application, 'tempSales', tempSales)
            application.save()

            setattr(logForUserBehavior, 'application', application)
            setattr(logForUserBehavior, 'full_name', User.objects.get(id=user_id).full_name)
            setattr(logForUserBehavior, 'user', User.objects.get(id=user_id))
            logForUserBehavior.save()
            return tempSales
    @atomic
    def update(self, instance, validated_data):
        user_id = self.context['request'].user.id
        logForUserBehavior = LogForUserBehavior()
        application = Application.objects.get(id=self.context['request'].data['app'])
        if application.use == "修改資料":
            validated_data.pop('sales_id', None)
        else:
            roc_sales = Sales.objects.filter(roc_id=validated_data.get('roc_id', None))
            if roc_sales:
                for i in roc_sales:
                    if i.state == "在職":
                        raise serializers.ValidationError('重複的身分證字號')
                    else:
                        if not (i.date_of_leave < (datetime.now().date() - timedelta(days=6*30))) :
                            print((str)(validated_data.get('sales_unit', None)))
                            if (str)(i.sales_unit_id) != (str)(validated_data.get('sales_unit', None)):
                                raise serializers.ValidationError('離職後半年欲復職需在同單位')
        for i in validated_data:
            setattr(instance, i, validated_data.get(i, getattr(instance, i)))
        if self.context['request'].data['application'] == "deliver":
            application.status = "待審核"
            application.modifiedText = self.context['request'].data['reason']
            if application.use == "新增業務":
                if not instance.date_of_come:
                    setattr(instance, 'date_of_come', datetime.now(pytz.timezone('Asia/Taipei')).date())
                application.unit = validated_data.get('sales_unit', getattr(instance, 'sales_unit'))
        elif self.context['request'].data['application'] == "save":
            if application.use == "新增業務":
                application.unit = validated_data.get('sales_unit', getattr(instance, 'sales_unit'))
        application.save()


        setattr(logForUserBehavior, 'behavior',application.use + "-" + application.status)
        setattr(logForUserBehavior, 'application', application)
        setattr(logForUserBehavior, 'full_name', User.objects.get(id=user_id).full_name)
        setattr(logForUserBehavior, 'user', User.objects.get(id=user_id))
        logForUserBehavior.save()
        instance.save()
        return instance

class TempSalesViewSerializer(serializers.ModelSerializer):
    class Meta:
        model = TempSales
        fields = '__all__'
class TempSalesBasicInfoSerializer(serializers.ModelSerializer):

    class Meta:
        model = TempSales
        fields = ('sales_id', 'company', 'id', 'full_name', 'work_type', 'position')

class TempSalesBankInfoSerializer(serializers.ModelSerializer):

    class Meta:
        model = TempSales
        fields = ('bank_num', 'bank_pri_name', 'bank_sub_name', 'id', 'full_name', 'bank_account')
    # def create(self, validated_data):
    #     print(validated_data)
    #     test = Test(
    #             created = validated_data.get('created', None),
    #             user = validated_data.get('user', None),
    #             title = validated_data.get('title', None),
    #             img1 = validated_data.get('img1', None),
    #             img1_thumb = validated_data.get('img1_thumb', None),
    #             img1_datetime = validated_data.get('img1_datetime', None),
    #             img2 = validated_data.get('img2', None),
    #             img2_datetime = validated_data.get('img2_datetime', None),
    #         )
    #     test.save(force_insert=True ,force_update=True, api='Create')
    #     return test
    #
    # def update(self, instance, validated_data):
    #     print(validated_data)
    #     instance.user = validated_data.get('user', instance.user)
    #     instance.title = validated_data.get('title', instance.title)
    #     instance.img1 = validated_data.get('img1', instance.img1)
    #     instance.img1_thumb = validated_data.get('img1_thumb', instance.img1_thumb)
    #     instance.img1_datetime = validated_data.get('img1_datetime', instance.img1_datetime)
    #     instance.img2 = validated_data.get('img2', instance.img2)
    #     instance.img2_datetime = validated_data.get('img2_datetime', instance.img2_datetime)
    #     instance.save(force_insert=False ,force_update=True, api='Update')
    #     return instance
